#   Code to produce code for Fidelity date widget
#
from rlextra.forms.qwidget import QWidget

def jsDateArgsFromIsoDate(text):
    # yr, mon, day, hrs, mins, sec, msec
    if len(text) != 8:
        raise ValueError('date string must be in format YYYYMMDD')
    return (int(text[:4]), int(text[4:6])-1, int(text[6:]), 0, 0, 0, 0)

class Date_Widget(QWidget):
    def __init__(self,**kw):
        QWidget.__init__(self,
                _kw=kw,
                name='date',
                editable=1,
                allowEmpty=1,
                label="date",
                errorMessage="Please enter a valid date",
                shortHelp="",
                longHelp="",
                httpDir = '/res',
                jsFileName = 'date_base',
                dd='',
                mm='',
                yyyy='',
                yyyymmdd='',
                sampleData='',
                min='19000101',#(1900, 1, 1, 0,0,0,0,0,-1),
                max='21000101',#(2100, 1, 1, 0,0,0,0,0,-1),
                rangeErrorMessage='',
                )

    def __getattr__(self, name):
        if name == 'value':
            return self.yyyymmdd
        else:
            return QWidget.__getattr__(self, name)

    def __setattr__(self, name, value):
        if name == 'value':
            if not value:
                return
            yyyy, mm, dd = value[:4], value[4:6], value[6:]
            self.dd = dd
            self.mm = mm
            self.yyyy = yyyy
            self.yyyymmdd = value
            self.setChildDict(('dd','mm','yyyy','yyyymmdd'))
        else:
            self.__dict__[name] = value

    def postInit(self):
        QWidget.postInit(self)
        #integer attribs
        for n in ('allowEmpty',):
            setattr(self,n,int(getattr(self,n)))
        self._get_pfxname()
        self.form = 'document.forms[0]' # this is what we assume
        self.widname = self.name.replace('.','_')+'_DW'
        
        self.setChildDict(('dd','mm','yyyy','yyyymmdd'))
        opts = 0
        for n,d in (('allowEmpty',1),):
            opts |= getattr(self,n,0) and d or 0
        self._flags = opts

    def getDText(self):
        dText=[]
        def d(txt,a=dText.append,fmt=self._fmt):
            a(fmt(txt))
        self._v_yyyymmdd = self.childDict['yyyymmdd']
        self._v_yyyy = self.childDict['yyyy']
        self._v_mm = self.childDict['mm']
        self._v_dd = self.childDict['dd']
        d('<tr><td>%(label)s<input type="hidden" name="%(_pfxname)syyyymmdd" value="%(_v_yyyymmdd)s"></td>')
        if self.editable:
            d('<td nowrap><input type="text" size="2" maxlength="2" name="%(_pfxname)sdd" >')
            d('&nbsp;/&nbsp;<input type="text" size="2" maxlength="2" name="%(_pfxname)smm">')
            d('&nbsp;/&nbsp;<input type="text" size="4" maxlength="4" name="%(_pfxname)syyyy"></td>')
            d('<td align="right">%s</td></tr>' % self._getDotHelp())
            #d('<td><b><p tooltip="%(longHelp)s" ttclass="tooltip">Help</p></b></td></tr>')
        else:
            d('<td nowrap>%(_v_dd)s/%(_v_mm)s/%(_v_yyyy)s')
            d('<input type="hidden" name="%(_pfxname)sdd">')
            d('<input type="hidden" name="%(_pfxname)smm">')
            d('<input type="hidden" name="%(_pfxname)syyyy"></td>')
        self.dText = '\n'.join(dText)
        return self.dText

    def getIText(self):
        if not self.editable:
            self.iText = ''
        else:

            self.iText = self._fmt('var %(widname)s = new RL_DATE("%(_pfxname)s","%(errorMessage)s","%(rangeErrorMessage)s",%(_flags)d, new Date%(min)s, new Date%(max)s);',
                                   min=jsDateArgsFromIsoDate(self.min), max=jsDateArgsFromIsoDate(self.max))
        return self.iText

    def getLText(self):
        if not self.editable:
            self.lText = ''
        else:
            self.lText = self._fmt('%(widname)s.onload();\n')
        return self.lText

    def getVText(self):
        if not self.editable:
            self.vText = ''
        else:
            self.vText = self._fmt('%(widname)s.formLevelValidate()')
        return self.vText

if __name__=='__main__': #noruntests
    from rlextra.utils.buildutils import getFlag
    def boolFlag(name,d=0,has=1):
        return (getFlag((has and '--has' or '')+name,0,d) and not getFlag('--no'+name)) and 1 or 0

    if getFlag('--saveAFiles'):
        Date_Widget().saveAFile()
    else:
        Date_Widget(
            httpDir = getFlag('--httpDir',1,'/wefbranding'),
            ).buildTestHTML(getFlag('--fn',1,'test_fs.html'))
