# Copyright ReportLab Europe Ltd. 2000-2004
# $Id $

import os
from reportlab.lib.utils import rl_isfile
from rlextra.ers.webapp import WebApp
from rlextra.forms.question import Questionnaire

class FileNotFoundError(Exception):
    pass

class QdfWebApp(WebApp):

    def __init__(self, **kw):
        import rlextra.forms        
        self.qdfDtdDir = os.path.dirname(rlextra.forms.__file__)

        self._qdfSearchPath = [os.getcwd()]
        if os.path.isdir('forms'):
            self._qdfSearchPath.append('forms')

        WebApp.__init__(self, **kw)

    def dispatch(self, request, response):
        self._setQuestionnaire(request)
        WebApp.dispatch(self, request, response)
    def dispatchWithoutSettingQuestionnaire(self, request, response):
        WebApp.dispatch(self, request, response)

    def _setQuestionnaire(self, request):
        params = request.params
        qdfName = params.get('qdf')
        if qdfName is not None:
            questionnaire = self.loadQuestionnaire(qdfName)
            questionnaire.setTextDictionary(params)
            if params.has_key('sample'):
                questionnaire.makeSample()
        else:
            questionnaire = None
        self.questionnaire = questionnaire

    def loadQuestionnaire(self, name):
        """Find a form on the path and load it.

        Return a questionnaire object, or raise an error if not found.

        """
        qdfFileName = name + '.qdf'
        fullFileName = None
        for searchDir in self._qdfSearchPath:
            if rl_isfile(searchDir + os.sep + qdfFileName):
                fullFileName = searchDir + os.sep + qdfFileName
                continue

        if not fullFileName:
            raise FileNotFoundError('Questionnaire file %s not found!' % qdfFileName)

        return self._createQuestionnaire(name, fullFileName)

    def _createQuestionnaire(self, formName, fullFileName):
        from rlextra.forms import question
        return question.Questionnaire(fullFileName)

    def _getPreppyTemplateBasename(self, formName):
        return 'template'

    def _resPathname(self, filename):
        return os.path.join('res', filename)

    def getResUrl(self, filename):
        import posixpath
        return posixpath.join(self.resUrl, filename)

    def _hiddenFields(self, request):
        return {}

    def _lhbCell(self):
        return ''

    def action_showQuestionnaire(self, request, response):

        """Displays a questionnaire in stages.

        The following parameters and modes are supported.

        qdf (required) - name of qdf file, without the extension.
            This app searches the rlextra\forms\test directory
            and the .\forms directory for files.

        submittedSection - name of section being submitted.  If not
            provided, the first display sections is shown.

        section - request a specified section, rather than letting
            the Questionnaire and WebApp determine the next section
            from the submitted section.  Takes precedence over
            submittedSection.

        allSections - display all sections at once.  Takes precedence
            over submittedSection and section.

        # XXX rename
        submitting - performing final submission.  In this case, we
            perform the finalAction after executing any standard
            Questionnaire hook methods.

        finalAction - the final action when the last section is
            submitted.

        submitCaption - submit label for Next button; if 'None', don't
            display a Next button

        sample - if present, canned sample values appear in the form.

        """
        from rlextra.preppy import preppy

        params = request.params
        qdf = params.get('qdf')
        section = params.get('section', [])
        allSections = params.get('allSections', 0)
        finalAction = params.get('finalAction', 'echo')
        submittedSection = params.get('submittedSection', [])
        submitting = params.get('submitting', 0)
        submitCaption = params.get('submitCaption', None)

        questionnaire = self.questionnaire

        if not submitting:
            if section:
                section = section.split(',')
            if submittedSection:
                submittedSection = submittedSection.split(',')
            if allSections:
                section = Questionnaire.AllSections
            section, form = questionnaire.makeNextHtmlForm(
                request.getScriptURL(), finalAction, qdf,
                submittedSection, section,
                submitCaption=submitCaption,
                hiddenFields=self._hiddenFields(request),
                lhbcell=self._lhbCell(),
                )

            if form is None:
                # Somebody hid all sections from submittedSection up to
                # the end of the questionnaire.
                submitting = 1
            else:
                params = {
                    'JSV': questionnaire.JSV,
                    'form': form,
                    'app': self,
                    }
                m = preppy.getModule(self._resPathname(self._getPreppyTemplateBasename(qdf)))
                response.write(m.getOutput(params))

        if submitting:
            request.params['action'] = finalAction
            self._handleRequest(request, response)
